/****************************************************************************************** 
 *	Chili DirectX Framework Version 16.07.20											  *	
 *	Game.cpp																			  *
 *	Copyright 2016 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "MainWindow.h"
#include "Game.h"
#include "Colors.h"
#include "Game.h"
#include "Button.h"
#include <stdexcept>

Game::Game(MainWindow& wind)
	:
	wnd(wind),
	gfx(wind),
	rng(std::random_device()()),
	galaxy(rng),
	font("Fixedsys16x28.bmp"),
	colonies(1250, 1500, 35, 750),
	ships(1250, 1500, 35, 750)
{
	int count = 0;
	std::vector<Planet> hp;
	for (int i = 0; i < GALAXY_SIZE; i++)
	{
		if (!galaxy.get_sector(i).planet.empty() && galaxy.get_sector(i).planet[0].is_habitable() == true)
		{
			hp.push_back(galaxy.get_sector(i).planet[0]);
			count++;
		}
	}
	if (count <= NUMPLAYERS)
		throw std::runtime_error("Infertile galaxy :o try again!");
	for (int j = 0; j < NUMPLAYERS; j++)
	{
		std::uniform_int_distribution<int> dist(0, count - j - 1);
		bool is_colonized = false;
		int roll = 0;
		do
		{
			is_colonized = false;
			roll = dist(rng);
			for (int p = 0; p < NUMPLAYERS; p++)
			{
				if (player[p].is_colony(hp[roll].get_loc()) == true)
					is_colonized = true;
			}
		} while (is_colonized == true);

		player[j].add_colony(hp[roll]);

		for (int i = 0; i < GALAXY_SIZE; i++)
		{
			if (abs(galaxy.get_sector(i).loc.x - hp[roll].get_loc().x) + abs(galaxy.get_sector(i).loc.y - hp[roll].get_loc().y) <= 2)
			{
				player[j].add_discovered(i);
				player[j].add_in_range(i);
				if (!galaxy.get_sector(i).planet.empty())
				{
					player[j].add_discovered_planet(galaxy.get_sector(i).planet[0]);
				}
			}
		}
	}
	player[0].set_colour(Colors::Cyan);
	player[1].set_colour(Colors::Magenta);
	print_planet = player[0].get_colony(0);
}

void Game::Go()
{
	gfx.BeginFrame();
	if (start_screen == false)
	{
		if (player[playernum].set_up == false)
		{
			setUpScreen(playernum);
		}
		else if (loaded == false)
		{
			load(playernum);
		}
		else
		{
			if (wnd.mouse.LeftIsPressed() && wnd.mouse.GetPosX() >= ENDTURN.left && wnd.mouse.GetPosX() <= ENDTURN.right && wnd.mouse.GetPosY() >= ENDTURN.top && wnd.mouse.GetPosY() <= ENDTURN.bottom)
			{
				playernum++;
				if (playernum == NUMPLAYERS)
					playernum = 0;
				print_planet = player[playernum].get_colony(0);
				start_screen = true;
				loaded = false;
			}
			gameScreen(playernum);
		}
	}
	else
	{
		startTurn(playernum);
	}
	gfx.EndFrame();
}

void Game::startTurn(int p)
{
	std::string text1 = "PLAYER ";
	std::string text2 = std::to_string(p + 1);
	std::string text3 = "START TURN";
	Button b1(711, 823, 365, 393, text1, player[p].get_colour());
	Button b2(823, 839, 365, 393, text2, player[p].get_colour());
	Button b3(695, 855, 407, 435, text3, Colors::White);
	b1.draw(gfx, font);
	b2.draw(gfx, font);
	b3.draw(gfx, font);
	if (b3.is_clicked(wnd.mouse) == true)
	{
		b3.draw_outline(gfx);
		start_screen = false;
	}
}

void Game::setUpScreen(int p)
{
	drawGalaxy(p);
	std::string num = std::to_string(p + 1);
	std::string text = "PLAYER ";

	Location loc;
	loc.x = GALAXY_WIDTH * SECTOR_DIMENSION;
	loc.y = SCREEN_HEIGHT - font.get_glyphHeight() - 1;

	font.drawText(text, loc, gfx, player[p].get_colour());

	loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + font.get_glyphWidth() * 7;
	font.drawText(num, loc, gfx, player[p].get_colour());

	if (player[p].first_city == false)
	{

		text = "CHOOSE CITY";
		loc.x = SCREEN_WIDTH - 12 * font.get_glyphWidth();
		loc.y = SCREEN_HEIGHT - 70;
		Button b1(loc.x, loc.x + 11 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		b1.draw(gfx, font);
		text = "LOCATION";
		loc.x = SCREEN_WIDTH - 10 * font.get_glyphWidth();
		loc.y = SCREEN_HEIGHT - 28;
		Button b2(loc.x, loc.x + 11 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		b2.draw(gfx, font);
		if (print_planet.get_loc() == player[p].get_colony(0).get_loc())
		{
			if (wnd.mouse.LeftIsPressed() && wnd.mouse.GetPosX() >= PLANET.left && wnd.mouse.GetPosX() <= PLANET.right && wnd.mouse.GetPosY() >= PLANET.top && wnd.mouse.GetPosY() <= PLANET.bottom)
			{
				int y = wnd.mouse.GetPosY() / 100;
				int x = (wnd.mouse.GetPosX() - GALAXY_WIDTH*SECTOR_DIMENSION) / 100;
				player[p].build_improvement(player[p].get_colony(0).get_loc(), 3 * y + x, 'C');
				player[p].first_city = true;
				print_planet = player[p].get_colony(0);
				for (int i = 0; i < 3; i++)
				{
					player[p].add_person(player[p].get_colony(0).get_loc(), 3 * y + x, rng);
				}
			}
		}
	}
	else if (player[p].first_farm == false)
	{
		text = "CHOOSE FARM";
		loc.x = SCREEN_WIDTH - 12 * font.get_glyphWidth();
		loc.y = SCREEN_HEIGHT - 70;
		font.drawText(text, loc, gfx, Colors::White);
		text = "LOCATION";
		loc.x = SCREEN_WIDTH - 10 * font.get_glyphWidth();
		loc.y = SCREEN_HEIGHT - 28;
		font.drawText(text, loc, gfx, Colors::White);
		if (print_planet.get_loc() == player[p].get_colony(0).get_loc())
		{
			if (wnd.mouse.LeftIsPressed() && wnd.mouse.GetPosX() >= PLANET.left && wnd.mouse.GetPosX() <= PLANET.right && wnd.mouse.GetPosY() >= PLANET.top && wnd.mouse.GetPosY() <= PLANET.bottom)
			{
				int y = wnd.mouse.GetPosY() / 100;
				int x = (wnd.mouse.GetPosX() - GALAXY_WIDTH*SECTOR_DIMENSION) / 100;
				if (player[p].get_colony(0).is_beside(3 * y + x))
				{
					player[p].build_improvement(player[p].get_colony(0).get_loc(), 3 * y + x, 'F');
					player[p].first_farm = true;
					print_planet = player[p].get_colony(0);
				}
			}
		}
	}
	else if (player[p].person1 == false||player[p].person2==false||player[p].person3==false)
	{
		std::string text = "Person 1";
		Location loc{};
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 70;
		loc.y = 314;
		Button p1(loc.x, loc.x + 8 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		p1.draw(gfx, font);
		text = "Ability = ";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 54;
		loc.y += 28 + 7;
		font.drawText(text, loc, gfx, Colors::White);
		text = std::to_string(player[p].get_person(1).ability);
		loc.x += 10 * 16;
		font.drawText(text, loc, gfx, Colors::White);

		text = "Person 2";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 70;
		loc.y += 28 + 14;
		Button p2(loc.x, loc.x + 8 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		p2.draw(gfx, font);
		text = "Ability = ";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 54;
		loc.y += 28 + 7;
		font.drawText(text, loc, gfx, Colors::White);
		text = std::to_string(player[p].get_person(2).ability);
		loc.x += 10 * 16;
		font.drawText(text, loc, gfx, Colors::White);

		text = "Person 3";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 70;
		loc.y += 28 + 14;
		Button p3(loc.x, loc.x + 8 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		p3.draw(gfx, font);
		text = "Ability = ";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 54;
		loc.y += 28 + 7;
		font.drawText(text, loc, gfx, Colors::White);
		text = std::to_string(player[p].get_person(3).ability);
		loc.x += 10 * 16;
		font.drawText(text, loc, gfx, Colors::White);

		text = "Fmr";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 68;
		loc.y += 28 + 14 + 28;
		Button Fmr(loc.x, loc.x + 3 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Fmr.draw(gfx, font);
		text = "Eng";
		loc.x += 3 * font.get_glyphWidth() + 68;
		Button Eng(loc.x, loc.x + 3 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Eng.draw(gfx, font);
		text = "Sci";
		loc.x= GALAXY_WIDTH * SECTOR_DIMENSION + 68;
		loc.y += 28 + 14;
		Button Sci(loc.x, loc.x + 3 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Sci.draw(gfx, font);
		text = "Tch";
		loc.x += 3 * font.get_glyphWidth() + 68;
		Button Tch(loc.x, loc.x + 3 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Tch.draw(gfx, font);
		text = "Mnr";
		loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + 68;
		loc.y += 28 + 14;
		Button Mnr(loc.x, loc.x + 3 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Mnr.draw(gfx, font);
		text = "Sdr";
		loc.x += 3 * font.get_glyphWidth() + 68;
		Button Sdr(loc.x, loc.x + 3 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Sdr.draw(gfx, font);

		text = "CHOOSE";
		loc.x = SCREEN_WIDTH - 8 * 16;
		loc.y = SCREEN_HEIGHT - 28 - 7 - 28;
		font.drawText(text, loc, gfx, Colors::White);
		text = "JOBS";
		loc.x = SCREEN_WIDTH - 7 * 16;
		loc.y += 28 + 7;
		font.drawText(text, loc, gfx, Colors::White);

		if (Fmr.is_clicked(wnd.mouse))
		{
			Fmr.draw_outline(gfx);
			job = 'f';
		}
		if (Eng.is_clicked(wnd.mouse))
		{
			Eng.draw_outline(gfx);
			job = 'e';
		}
		if (Sci.is_clicked(wnd.mouse))
		{
			Sci.draw_outline(gfx);
			job = 'r';
		}
		if (Tch.is_clicked(wnd.mouse))
		{
			Tch.draw_outline(gfx);
			job = 't';
		}
		if (Mnr.is_clicked(wnd.mouse))
		{
			Mnr.draw_outline(gfx);
			job = 'm';
		}
		if (Sdr.is_clicked(wnd.mouse))
		{
			Sdr.draw_outline(gfx);
			job = 's';
		}

		if (player[p].person1 == false)
		{
			p1.draw_outline(gfx);
			if (p1.is_clicked(wnd.mouse) && job != ' ')
			{
				player[p].set_job(1, job);
				job = ' ';
				player[p].person1 = true;
			}
		}
		else if (player[p].person2 == false)
		{
			p2.draw_outline(gfx);
			if (p2.is_clicked(wnd.mouse) && job != ' ')
			{
				player[p].set_job(2, job);
				job = ' ';
				player[p].person2 = true;
			}
		}
		else if (player[p].person3 == false)
		{
			p3.draw_outline(gfx);
			if (p3.is_clicked(wnd.mouse) && job != ' ')
			{
				player[p].set_job(3, job);
				job = ' ';
				player[p].person3 = true;
			}
		}
	}
	else if (player[p].start_project == false)
	{
		std::string text = "Farm";
		Location loc{ GALAXY_WIDTH * SECTOR_DIMENSION + 70,314 };
		Button Farm(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Farm.draw(gfx, font);
		text = "City";
		loc.y += 28 + 14;
		Button City(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		City.draw(gfx, font);
		text = "Mine";
		loc.y += 28 + 14;
		Button Mine(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Mine.draw(gfx, font);
		text = "Ship";
		loc.y += 28 + 14;
		Button Ship(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Ship.draw(gfx, font);
		text = "School";
		loc.y += 28 + 14;
		Button School(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		School.draw(gfx, font);
		text = "Laborotory";
		loc.y += 28 + 14;
		Button Lab(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Lab.draw(gfx, font);
		text = "Work plant";
		loc.y += 28 + 14;
		Button Work(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Work.draw(gfx, font);
		text = "Barracks";
		loc.y += 28 + 14;
		Button Barracks(loc.x, loc.x + 10 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
		Barracks.draw(gfx, font);

		if (Farm.is_clicked(wnd.mouse))
		{
			Farm.draw_outline(gfx);
			project = Projects::farm;
		}
		if (City.is_clicked(wnd.mouse))
		{
			City.draw_outline(gfx);
			project = Projects::city;
		}
		if (Mine.is_clicked(wnd.mouse))
		{
			Mine.draw_outline(gfx);
			project = Projects::mine;
		}
		if (Ship.is_clicked(wnd.mouse))
		{
			Ship.draw_outline(gfx);
			project = Projects::ship;
		}
		if (School.is_clicked(wnd.mouse))
		{
			School.draw_outline(gfx);
			project = Projects::school;
		}
		if (Lab.is_clicked(wnd.mouse))
		{
			Lab.draw_outline(gfx);
			project = Projects::lab;
		}
		if (Work.is_clicked(wnd.mouse))
		{
			Work.draw_outline(gfx);
			project = Projects::workplant;
		}
		if (Barracks.is_clicked(wnd.mouse))
		{
			Barracks.draw_outline(gfx);
			project = Projects::barracks;
		}

		if (project.type!=' ')
		{
			text = "OK";
			loc.y += 28 + 28;
			loc.x += 16 * 5;
			Button ok(loc.x, loc.x + 2 * font.get_glyphWidth(), loc.y, loc.y + font.get_glyphHeight(), text, Colors::White);
			ok.draw(gfx, font);

			if (ok.is_clicked(wnd.mouse))
			{
				ok.draw_outline(gfx);
				player[p].set_project(player[p].get_colony(0).get_loc(), project);
				project = Projects::start;
				player[p].start_project = true;
			}
		}

		text = "CHOOSE";
		loc.x = SCREEN_WIDTH - 8 * 16;
		loc.y = SCREEN_HEIGHT - 28 - 7 - 28;
		font.drawText(text, loc, gfx, Colors::White);
		text = "PROJECT";
		loc.x = SCREEN_WIDTH - 8 * 16;
		loc.y += 28 + 7;
		font.drawText(text, loc, gfx, Colors::White);
	}
	else
	{
		player[p].set_up_food();
		text = "END TURN";
		font.drawText(text, { ENDTURN.left,ENDTURN.top }, gfx, Colors::White);
		if (wnd.mouse.LeftIsPressed() && wnd.mouse.GetPosX() >= ENDTURN.left && wnd.mouse.GetPosX() <= ENDTURN.right && wnd.mouse.GetPosY() >= ENDTURN.top && wnd.mouse.GetPosY() <= ENDTURN.bottom)
		{
			player[p].set_up = true;
		}
	}

	if (wnd.mouse.LeftIsPressed() && player[p].is_discovered_planet({ wnd.mouse.GetPosX() / SECTOR_DIMENSION,wnd.mouse.GetPosY() / SECTOR_DIMENSION }))
	{
		print_planet = player[p].get_planet({ wnd.mouse.GetPosX() / SECTOR_DIMENSION,wnd.mouse.GetPosY() / SECTOR_DIMENSION });
	}

	if (player[p].is_colony(print_planet.get_loc()))
	{
		print_planet.print_colony(gfx, font);
	}
	else
	{
		print_planet.print(gfx);
	}
}

void Game::gameScreen(int p)
{
	Button colony_menu(1231, 1359, 0, 28, "Colonies", Colors::White);
	colony_menu.draw(gfx, font);
	Button ship_menu(1389, 1469, 0, 28, "Ships", Colors::White);
	ship_menu.draw(gfx, font);

	if (colony_menu.is_clicked(wnd.mouse))
	{
		view_planet_list = true;
		view_ship_list = false;
	}
	if (ship_menu.is_clicked(wnd.mouse))
	{
		view_planet_list = false;
		view_ship_list = true;
	}
	if (wnd.mouse.LeftIsPressed() && player[p].is_discovered_planet({ wnd.mouse.GetPosX() / SECTOR_DIMENSION,wnd.mouse.GetPosY() / SECTOR_DIMENSION }))
	{
		view_planet_list = false;
		view_ship_list = false;
		print_planet = player[p].get_planet({ wnd.mouse.GetPosX() / SECTOR_DIMENSION,wnd.mouse.GetPosY() / SECTOR_DIMENSION });
	}
	drawGalaxy(p);
	std::string num = std::to_string(p + 1);
	std::string text = "PLAYER ";

	Location loc;
	loc.x = GALAXY_WIDTH * SECTOR_DIMENSION;
	loc.y = SCREEN_HEIGHT - font.get_glyphHeight() - 1;

	font.drawText(text, loc, gfx, player[p].get_colour());

	loc.x = GALAXY_WIDTH * SECTOR_DIMENSION + font.get_glyphWidth() * 7;
	font.drawText(num, loc, gfx, player[p].get_colour());

	text = "END TURN";
	font.drawText(text, { ENDTURN.left,ENDTURN.top }, gfx, Colors::White);


	if (view_planet_list == false && view_ship_list == false)
	{
		if (player[p].is_colony(print_planet.get_loc()))
		{
			print_planet.print_colony({ 0,28 },gfx, font);
		}
		else
		{
			print_planet.print({ 0,28 },gfx);
		}
	}
	if (view_planet_list == true)
	{
		if (view_action_list == false)
		{
			colonies.print(gfx, font);
		}
		else
		{
			for (auto& i : player[p].colony)
			{
				if (i.get_loc()==list_loc)
				{
					i.action_list.print(gfx, font);
				}
			}
		}
		for (auto& i : colonies.button)
		{
			if (i.is_clicked(wnd.mouse))
			{
				list_loc = i.loc;
				view_action_list = true;
			}
		}
	}
	if (view_ship_list == true)
	{
		if (view_action_list == false)
		{
			ships.print(gfx, font);
		}
		else
		{
			for (auto& i : player[p].ship)
			{
				if (i.get_loc() == list_loc)
				{
					i.action_list.print(gfx, font);
				}
			}
		}
		for (auto& i : ships.button)
		{
			if (i.is_clicked(wnd.mouse))
			{
				list_loc = i.loc;
				view_action_list = true;
			}
		}
	}
}

void Game::load(int p)
{
	for (auto& i :player[p].colony)
	{
		i.update_food();
		i.update_production();
		i.check_reeducate();
		i.check_mining();
	}
	player[p].births(rng);
	for (auto& i:player[p].ship)
	{
		i.update_food();
		int speed = 0;
		if (player[p].get_tech(WARP_DRIVE) >= 20 && i.get_warp_engines() >= 3)
			speed = 5;
		else if (player[p].get_tech(WARP_DRIVE) >= 15 && i.get_warp_engines() >= 2)
			speed = 3;
		else if (player[p].get_tech(WARP_DRIVE) >= 10 && i.get_warp_engines() >= 1)
			speed = 2;
		else
			speed = 1;
		i.add_action("Move", false, speed);
		if (i.get_engineering() > 0 && i.get_health() < 50 && is_beside_colony(i))
			i.add_action("Heal", false, 1);
		if (is_beside_colony(i))
			i.add_action("Load", false, NO_LIMIT);
		if (is_beside_colony(i)&&!i.is_empty())
			i.add_action("Unload", false, NO_LIMIT);
	}

	for (auto& i:player[p].colony)
	{
		Color colour;
		if (!i.action.empty())
			colour = Colors::Red;
		else
			colour = Colors::White;
		std::string text1 = "x: ";
		std::string text2 = std::to_string(i.get_loc().x);
		std::string text3 = " y: ";
		std::string text4 = std::to_string(i.get_loc().y);
		std::string text = text1 + text2 + text3 + text4;
		colonies.add_button(text, colour,i.get_loc());
	}
	for (auto& i:player[p].ship)
	{
		Color colour;
		if (!i.action.empty())
			colour = Colors::Red;
		else
			colour = Colors::White;
		std::string text1 = "x: ";
		std::string text2 = std::to_string(i.get_loc().x);
		std::string text3 = " y: ";
		std::string text4 = std::to_string(i.get_loc().y);
		std::string text = text1 + text2 + text3 + text4;
		ships.add_button(text, colour,i.get_loc());
	}
	loaded = true;
}

void Game::drawGalaxy(int p)
{
	Location loc = { 0,0 };
	for (int i = 0; i < GALAXY_SIZE; i++)
	{
		loc.x = (i % GALAXY_WIDTH) * SECTOR_DIMENSION;
		loc.y = (i / GALAXY_WIDTH) * SECTOR_DIMENSION;
		if (galaxy.get_sector(i).star == true)
		{
			gfx.DrawRectDim(loc.x,loc.y,SECTOR_DIMENSION,SECTOR_DIMENSION, Colors::White);
		}
		else if (player[p].is_discovered(i) == true)
		{
			if (!galaxy.get_sector(i).planet.empty())
			{
				gfx.DrawRectDim(loc.x, loc.y, SECTOR_DIMENSION, SECTOR_DIMENSION, galaxy.get_sector(i).planet[0].get_colour());
			}
			else
			{
				if (player[p].is_in_range(i) == true)
				{
					gfx.DrawRectDim(loc.x, loc.y, SECTOR_DIMENSION, SECTOR_DIMENSION, Colors::LightGray);
				}
				else
				{
					gfx.DrawRectDim(loc.x, loc.y, SECTOR_DIMENSION, SECTOR_DIMENSION, Colors::Gray);
				}
			}
		}
	}

	for (int i = 0; i < GALAXY_HEIGHT; i++)
	{
		gfx.PutPixel(GALAXY_WIDTH * SECTOR_DIMENSION, i, Colors::White);
	}
}

bool Game::is_beside_colony(Ship& s)
{
	int i = s.get_loc().y * GALAXY_WIDTH + s.get_loc().x;
	if (i == 0)
	{
		if (!galaxy.get_sector(1).planet.empty() && galaxy.get_sector(1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(GALAXY_WIDTH).planet.empty() && galaxy.get_sector(GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i == GALAXY_WIDTH - 1)
	{
		if (!galaxy.get_sector(GALAXY_WIDTH-2).planet.empty() && galaxy.get_sector(GALAXY_WIDTH-2).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(GALAXY_WIDTH*2-1).planet.empty() && galaxy.get_sector(GALAXY_WIDTH*2-1).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i == GALAXY_WIDTH * GALAXY_WIDTH - GALAXY_WIDTH)
	{
		if (!galaxy.get_sector(i-GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i-GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i+1).planet.empty() && galaxy.get_sector(i+1).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i == GALAXY_SIZE - 1)
	{
		if (!galaxy.get_sector(i-GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i-GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i-1).planet.empty() && galaxy.get_sector(i-1).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i / GALAXY_WIDTH == 0)
	{
		if (!galaxy.get_sector(i+1).planet.empty() && galaxy.get_sector(i+1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i-1).planet.empty() && galaxy.get_sector(i-1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i+GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i+GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i / GALAXY_WIDTH == (GALAXY_HEIGHT - 1))
	{
		if (!galaxy.get_sector(i + 1).planet.empty() && galaxy.get_sector(i + 1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i - 1).planet.empty() && galaxy.get_sector(i - 1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i - GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i - GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i % GALAXY_WIDTH == 0)
	{
		if (!galaxy.get_sector(i + 1).planet.empty() && galaxy.get_sector(i + 1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i - GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i - GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i + GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i + GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else if (i % GALAXY_WIDTH == (GALAXY_WIDTH - 1))
	{
		if (!galaxy.get_sector(i - GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i - GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i - 1).planet.empty() && galaxy.get_sector(i - 1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i + GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i + GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
	}
	else
	{
		if (!galaxy.get_sector(i + 1).planet.empty() && galaxy.get_sector(i + 1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i - 1).planet.empty() && galaxy.get_sector(i - 1).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i + GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i + GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
		if (!galaxy.get_sector(i - GALAXY_WIDTH).planet.empty() && galaxy.get_sector(i - GALAXY_WIDTH).planet[1].get_colour() == s.get_colour())
			return true;
	}
	return false;
}

