#include "Planet.h"
#include "Constants.h"
#include "Colors.h"
#include <cassert>
#include <stdexcept>

Planet::Planet()
	:
	action_list(1250, 1800, 70, 750)
{
	project = Projects::start;
	loc = EMPTY;
	colour = Colors::Black;
	habitable = false;
	for (int i = 0; i < 9; i++)
	{
		map[i].improvement = ' ';
		map[i].resource = ' ';
		map[i].yield = 0;
	}
}

Planet::Planet (Location location,std::mt19937& rng)
	:
	action_list(1250,1800,70,750)
{
	std::uniform_int_distribution<int> d2(1, 2);
	std::uniform_int_distribution<int> d3(1, 3);
	std::uniform_int_distribution<int> d6(1, 6);
	int resourcenum;
	loc = location;
	project = Projects::start;
	if (d2(rng) == 1)
	{
		colour = Colors::Green;
		habitable = true;
	}
	else
	{
		colour = Colors::Red;
		habitable = false;
	}
	for (int i = 0; i < 9; i++)
	{
		if (d2(rng) == 1)
			map[i].yield = 0;
		else if (d2(rng)==1)
			map[i].yield = 1;
		else if (d3(rng)==1)
			map[i].yield = 3;
		else
			map[i].yield = 2;
		map[i].improvement = ' ';
		if (d6(rng)==1)
		{
			resourcenum = d3(rng);
			if (resourcenum == 1)
				map[i].resource = 'U';
			else if (resourcenum == 2)
				map[i].resource = 'A';
			else
				map[i].resource = 'D';
		}
		else
			map[i].resource = ' ';
	}
}

int Planet::get_population()
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		count += map[i].person.size();
	}
	return count;
}

int Planet::get_engineering()
{
	int engineering = 0;
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator j = map[i].person.rbegin(); j != map[i].person.rend(); ++j)
		{
			if (j->job == 'e')
				engineering += j->ability;
		}
	}
	return engineering;
}

int Planet::get_defense()
{
	int defense = 0;
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator j = map[i].person.rbegin(); j != map[i].person.rend(); ++j)
		{
			if (j->job == 's')
				defense += j->ability;
		}
	}
	return defense;
}

bool Planet::is_habitable()
{
	return habitable;
}

void Planet::set_loc(Location newloc)
{
	loc.x = newloc.x;
	loc.y = newloc.y;
}

Location Planet::get_loc()
{
	return loc;
}

void Planet::set_habitable(bool h)
{
	habitable = h;
}

bool Planet::get_habitable()
{
	return habitable;
}

tile Planet::get_tile(int i)
{
	return map[i];
}

int Planet::get_tile_yield(int i)
{
	return map[i].yield;
}

void Planet::print(Graphics& gfx)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100;
		newloc.y = (i / 3) * 100;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x,newloc.y,100,100, Colors::MakeRGB(255u,153u,0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u,128u,0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u,128u,0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u,51u,0u));
		}
	}
}

void Planet::print(Location loc1,Graphics& gfx)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100+loc1.x;
		newloc.y = (i / 3) * 100+loc1.y;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(255u, 153u, 0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u, 128u, 0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 128u, 0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 51u, 0u));
		}
	}
}

void Planet::print_colony(Graphics& gfx, Font& f)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100;
		newloc.y = (i / 3) * 100;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(255u, 153u, 0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u, 128u, 0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 128u, 0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 51u, 0u));
		}
		std::string c(1, map[i].improvement);
		std::string c2(1, map[i].resource);
		Color resource_colour;
		if (map[i].resource == 'U')
			resource_colour = Colors::Green;
		else if (map[i].resource == 'A')
			resource_colour = Colors::Yellow;
		else if (map[i].resource == 'D')
			resource_colour = Colors::MakeRGB(102u, 0u, 102u);
		if (map[i].resource == ' ' && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 42,newloc.y + 36 }, gfx, Colors::Black);
		}
		else if (map[i].resource != ' ' && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 23,newloc.y + 36 }, gfx, Colors::Black);
			f.drawText(c2, { newloc.x + 61,newloc.y + 36 }, gfx, resource_colour);
		}
		else if (map[i].resource != ' ' && map[i].improvement == ' ')
		{
			f.drawText(c2, { newloc.x + 42,newloc.y + 36 }, gfx, resource_colour);
		}
	}

}

void Planet::print_colony(Location loc1,Graphics& gfx, Font& f)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100 + loc1.x;
		newloc.y = (i / 3) * 100 + loc1.y;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(255u, 153u, 0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u, 128u, 0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 128u, 0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 51u, 0u));
		}
		std::string c(1, map[i].improvement);
		std::string c2(1, map[i].resource);
		Color resource_colour;
		if (map[i].resource == 'U')
			resource_colour = Colors::Green;
		else if (map[i].resource == 'A')
			resource_colour = Colors::Yellow;
		else if (map[i].resource == 'D')
			resource_colour = Colors::MakeRGB(102u, 0u, 102u);
		if (map[i].resource == ' ' && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 42,newloc.y + 36 }, gfx, Colors::Black);
		}
		else if (map[i].resource != ' ' && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 23,newloc.y + 36 }, gfx, Colors::Black);
			f.drawText(c2, { newloc.x + 61,newloc.y + 36 }, gfx, resource_colour);
		}
		else if (map[i].resource != ' ' && map[i].improvement == ' ')
		{
			f.drawText(c2, { newloc.x + 42,newloc.y + 36 }, gfx, resource_colour);
		}
	}

}

void Planet::build_improvement(int i, char c)
{
	map[i].improvement = c;
}

bool Planet::is_beside(int i)
{
	if (i == 0)
	{
		if (map[1].improvement != ' ')
			return true;
		if (map[3].improvement != ' ')
			return true;
	}
	if (i == 1)
	{
		if (map[0].improvement != ' ')
			return true;
		if (map[2].improvement != ' ')
			return true;
		if (map[4].improvement != ' ')
			return true;
	}
	if (i == 2)
	{
		if (map[1].improvement != ' ')
			return true;
		if (map[5].improvement != ' ')
			return true;
	}
	if (i == 3)
	{
		if (map[0].improvement != ' ')
			return true;
		if (map[4].improvement != ' ')
			return true;
		if (map[6].improvement != ' ')
			return true;
	}
	if (i == 4)
	{
		if (map[1].improvement != ' ')
			return true;
		if (map[3].improvement != ' ')
			return true;
		if (map[5].improvement != ' ')
			return true;
		if (map[7].improvement != ' ')
			return true;
	}
	if (i == 5)
	{
		if (map[2].improvement != ' ' || map[8].improvement != ' ' || map[4].improvement != ' ')
			return true;
	}
	if (i == 6)
	{
		if (map[3].improvement != ' ' || map[7].improvement != ' ')
			return true;
	}
	if (i == 7)
	{
		if (map[4].improvement != ' ' || map[6].improvement != ' ' || map[8].improvement != ' ')
			return true;
	}
	if (i == 8)
	{
		if (map[5].improvement != ' ' || map[7].improvement != ' ')
			return true;
	}
	return false;
}

void Planet::add_person(int i, int id, std::mt19937& rng)
{
	assert(i >= 0);
	assert(i < 9);
	std::uniform_int_distribution<int> d6(1, 6);
	Person person;
	person.job = ' ';
	person.number = id;
	person.reeducations = 0;
	person.ability = d6(rng);
	person.active = true;
	map[i].person.push_back(person);
}

bool Planet::is_person(int id)
{
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->number == id)
				return true;
		}
	}
	return false;
}

Person Planet::get_person(int id)
{
	assert(is_person(id));
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->number == id)
				return *p;
		}
	}
	throw std::runtime_error("no such person");
	Person o{};
	return o;
}

void Planet::set_job(int id, char j)
{
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->number == id)
				p->job = j;
		}
	}
}

void Planet::set_project(Project p)
{
	project = p;
}

void Planet::add_action(std::string t, bool n, int num)
{
	Action a(t, n, num);
	action.push_back(a);
	Color c = Colors::White;
	if (n == true)
		c = Colors::Red;
	action_list.add_button(t, c,a);
}

void Planet::remove_action(int i)
{
	assert(i != 0);
	assert(i < action.size());
	if (i == (action.size()-1))
	{
		action.pop_back();
	}
	if (i < (action.size()-1))
	{
		action.erase(action.begin() + i);
	}
}

Color Planet::get_colour()
{
	return colour;
}

int Planet::get_food()
{
	return food;
}

int Planet::get_food_output()
{
	std::vector<int> farmer;
	std::vector<int> farm;
	int food_output = 0;
	int num_farmers = 0;
	int num_farms = 0;
	int stop = 0;
	for (int i = 0; i < 9; i++)
	{
		if (map[i].improvement == 'F')
		{
			farm.push_back(map[i].yield);
			num_farms++;
		}
		for (std::vector<Person>::reverse_iterator j = map[i].person.rbegin(); j != map[i].person.rend(); ++j)
		{
			if (j->job == 'f')
			{
				farmer.push_back(j->ability);
				num_farmers++;
			}
		}
	}
	std::sort(farmer.begin(), farmer.begin() + num_farmers);
	std::sort(farm.begin(), farm.begin() + num_farms);
	if (num_farms > num_farmers)
		stop = num_farmers;
	else
		stop = num_farms;
	for (int i = 0; i < stop; i++)
	{
		food_output += farm[i] * farmer[i];
	}
	return food_output;
}

void Planet::update_food()
{
	food+= get_food_output() - get_population();
	if (food < 0)
		add_action("Starvation", true, abs(food));
}

void Planet::update_production()
{
	production_points += get_engineering();
	if (production_points >= project.cost)
	{
		production_points -= project.cost;
		add_action("Choose project", true,1);
		if (project.improvement)
			add_action("Place improvement", true,1);
		else if (project.type == 's')
			add_action("Place Ship", true,1);
		else if (project.type == 'w')
			warp_engines++;
		else if (project.type == 'a')
			antimatter_bombs++;
		else if (project.type == 'n')
			nuclear_missiles++;
	}
}

bool Planet::is_teacher()
{
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->job == 't' && p->active == true)
				return true;
		}
	}
	return false;
}

int Planet::num_active_teachers()
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->job == 't' && p->active == true)
				count++;
		}
	}
	return count;
}

int Planet::num_building(char type)
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		if (map[i].improvement == type)
			count++;
	}
	return count;
}

void Planet::check_reeducate()
{
	int count = 0;
	int num = 0;
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->reeducations > num_building('S'))
				count++;
		}
	}
	if (count > num_active_teachers())
		num = num_active_teachers();
	else
		num = count;
	add_action("Re-educate", false, num);
}

int Planet::num_miners()
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->job == 'm')
				count++;
		}
	}
	return count;
}

void Planet::check_mining()
{
	int num = 0;
	if (num_miners() > num_building('m'))
		num = num_building('m');
	else
		num = num_miners();
	if (num > 0)
		add_action("Mine", false, num);
}

void Planet::set_up_food()
{
	food += get_food_output();
}