#pragma once

#include "Location.h"
#include "Person.h"
#include <random>
#include "Graphics.h"
#include "Font.h"
#include "Action.h"
#include "Project.h"
#include "ActionList.h"

struct tile
{
	int yield;//0-3
	char improvement;//F-farm C-city M-megacity A-antimatter mine U-uranium mine D-dark matter mine S-school L-laborotory W-workplant B-barracks
	char resource;//u-uranium a-antimatter d-dark matter
	std::vector<Person> person;
};

class Planet
{
public:
	Planet();
	Planet(Location location,std::mt19937& rng);
	int get_population();
	int get_engineering();
	int get_defense();
	bool is_habitable();
	void set_loc(Location newloc);
	void set_habitable(bool h);
	Location get_loc();
	bool get_habitable();
	tile get_tile(int i);
	int get_tile_yield(int i);
	void print(Graphics& gfx);
	void print(Location loc1,Graphics& gfx);
	void print_colony(Graphics& gfx, Font& f);
	void print_colony(Location loc1,Graphics& gfx, Font& f);
	void build_improvement(int i, char c);
	bool is_beside(int i);
	void add_person(int i, int id, std::mt19937& rng);
	bool is_person(int id);
	Person get_person(int id);
	void set_job(int id, char j);
	void set_project(Project p);
	void add_action(std::string t, bool n,int num);
	void remove_action(int i);
	Color get_colour();
	int get_food();
	int get_food_output();
	void update_food();
	void update_production();
	bool is_teacher();
	int num_active_teachers();
	int num_building(char type);
	void check_reeducate();
	int num_miners();
	void check_mining();
	void set_up_food();
public:
	std::vector<Action> action;
	ActionList action_list;
private:
	bool habitable;
	Location loc;
	Color colour;
	tile map[9];
	int food=0;
	int uranium=0;
	int antimatter=0;
	int dark_matter=0;
	int antimatter_bombs=0;
	int nuclear_missiles=0;
	int warp_engines=0;
	int production_points=0;
	Project project;
	//F-farm C-city M-megacity m-mine S-school L-laborotory W-workplant
	//B-barracks s-ship a-animatter bomb n-nuclear missile w-warp engine
	//t-terraform
};