#include "Player.h"

Player::Player()
{
	colour = Colors::Black;
	discovered.reserve(GALAXY_SIZE+1);
	in_range.reserve(GALAXY_SIZE+1);
	for (int i = 0; i < GALAXY_SIZE; i++)
	{
		discovered.push_back(false);
		in_range.push_back(false);
	}
	for (int i = 0; i < NUM_TECHS; i++)
	{
		tech[i] = 0;
	}
}

void Player::add_colony(Planet p)
{
	colony.push_back(p);
}

void Player::add_discovered(int i)
{
	discovered[i] = true;
}

void Player::add_in_range(int i)
{
	in_range[i] = true;
}

void Player::set_colour(Color c)
{
	colour = c;
}

Color Player::get_colour()
{
	return colour;
}

bool Player::is_colony(Location loc)
{
	bool is_match = false;
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->get_loc() == loc)
			is_match = true;
	}
	return is_match;
}

Planet Player::get_colony(int i)
{
	return colony[i];
}

bool Player::is_discovered(int i)
{
	return discovered[i];
}

bool Player::is_in_range(int i)
{
	return in_range[i];
}

void Player::add_discovered_planet(Planet p)
{
	discovered_planet.push_back(p);
}

bool Player::is_discovered_planet(Location loc)
{
	bool is_match = false;
	for (std::vector<Planet>::reverse_iterator i = discovered_planet.rbegin(); i != discovered_planet.rend(); ++i)
	{
		if (i->get_loc() == loc)
			is_match = true;
	}
	return is_match;
}

Planet Player::get_planet(Location loc)
{
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->get_loc() == loc)
			return *i;
	}
	for (std::vector<Planet>::reverse_iterator i = discovered_planet.rbegin(); i != discovered_planet.rend(); ++i)
	{
		if (i->get_loc() == loc)
			return *i;
	}
	Planet p;
	return p;
}

void Player::build_improvement(Location loc, int j, char c)
{
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->get_loc() == loc)
			i->build_improvement(j, c);
	}
}

void Player::add_person(Location loc, int j, std::mt19937& rng)
{
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->get_loc() == loc)
		{
			i->add_person(j, idnum, rng);
			idnum++;
		}
	}
}

int Player::get_idnum()
{
	return idnum;
}

void Player::increment_idnum()
{
	idnum++;
}

Person Player::get_person(int id)
{
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->is_person(id))
			return i->get_person(id);
	}
	Person o;
	return o;
}

void Player::set_job(int id, char j)
{
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->is_person(id))
			i->set_job(id, j);
	}
}

void Player::set_project(Location loc, Project p)
{
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		if (i->get_loc() == loc)
			i->set_project(p);
	}
}

const std::vector<Planet>::reverse_iterator& Player::get_colony_rbegin() 
{
	return colony.rbegin();
}

const std::vector<Planet>::reverse_iterator& Player::get_colony_rend()
{
	return colony.rend();
}

const std::vector<Ship>::reverse_iterator& Player::get_ship_rbegin()
{
	return ship.rbegin();
}

const std::vector<Ship>::reverse_iterator& Player::get_ship_rend()
{
	return ship.rend();
}

void Player::births(std::mt19937& rng)
{
	int num_births = 0;
	int num = 0;
	for (std::vector<Planet>::reverse_iterator i = colony.rbegin(); i != colony.rend(); ++i)
	{
		num_births = 0;
		for (int j = 0; j < 9; j++)
		{
			if ((i->get_tile(j).improvement == 'C' && i->get_tile(j).person.size() < 3) || (i->get_tile(j).improvement == 'M' && i->get_tile(j).person.size() > 5))
			{
				i->add_person(j, idnum, rng);
				idnum++;
				num_births++;
			}
		}
		if (num_births > 0)
		{
			i->add_action("Assign job", true, num_births);
		}
		if (i->num_active_teachers() > 0&&num_births>0)
		{
			if (num_births > i->num_active_teachers())
				num = i->num_active_teachers();
			else
				num = num_births;
			i->add_action("Educate person", false, num);
		}
	}
}

int Player::get_tech(int i)
{
	return tech[i];
}

void Player::set_up_food()
{
	colony[0].set_up_food();
}